<?php
/**
 * @file
 * The group_mind_messages_type class File.
 */

/**
 * Manages Messages Creation.
 */
class group_mind_messages {
  /**************************************************************************
   * Manage messages on all necessary drupal HOOK.
   **************************************************************************/

  /**
   * Used by group_mind_messages.module in hook_node_insert.
   *
   * @param object $node
   *   The created node.
   *
   * @see group_mind_messages.module
   */
  public function node_insert($node) {
    // For every other content type we're not doing anything special.
    // Just use the default new_content message. No mails.
    // If the node is published (not filtered by antispam),
    // create the message.
    if ($node->status == 1) {
      $this->create_message('new_content', $node);
    }
  }

  /**
   * Used by group_mind_messages.module in hook_node_update().
   *
   * @param object $node
   *   The updated node.
   *
   * @see group_mind_messages.module
   */
  public function node_update($node) {
    // The node wasn't published previously (probably filtered by antispam)
    // but now it is.
    if ($node->original->status == 0 && $node->status == 1) {
      // Create a message on the author's profile, send him an email and
      // create a message on the moderator profile (if needed).
      $this->messages_moderated('new_content', $node, TRUE);
    }
    elseif ($node->status == 1) {
      // The node is updated outside of a publish, let's see
      // it was moderated or edited by the author.
      $this->messages_moderated('update_content', $node);
    }
  }

  /**
   * Used by group_mind_messages.module in hook_comment_insert().
   *
   * @param object $comment
   *   The created comment.
   *
   * @see group_mind_messages.module
   */
  public function comment_insert($comment) {
    // The comment is publish.
    if ($comment->status == 1) {
      // Just create a message on the author's profile.
      $this->create_message('new_comment', $comment);
    }
  }

  /**
   * Used by group_mind_messages.module in hook_comment_update().
   *
   * @param object $comment
   *   The updated comment.
   *
   * @see group_mind_messages.module
   */
  public function comment_update($comment) {
    // The comment wasn't published previously (probably filtered by antispam)
    // but now it is.
    if ($comment->original->status == 0 && $comment->status == 1) {
      // Create a message on the author's profile, send him an email and
      // create a message on the moderator profile (if needed).
      $this->messages_moderated('new_comment', $comment, TRUE);
    }
    elseif ($comment->status == 1) {
      // The node is updated outside of a publish, let's see it was moderated
      // or edited by the author.
      $this->messages_moderated('update_comment', $comment);
    }
  }
  /**************************************************************************
   * Useful functions.
   **************************************************************************/

  /**
   * Wrapper to create and save messages.
   *
   * @param string $message_type
   *   The type of message to create.
   * @param object $node
   *   The node to attach the message to.
   * @param int $uid
   *   (optional) The uid to attach the message to.
   *   If not given, we use the node's author.
   *
   * @return object
   *   The new message object.
   */
  protected function create_message($message_type, $node, $uid = FALSE) {
    $message = message_create($message_type, array('uid' => $uid ? $uid : $node->uid));
    $wrapper = entity_metadata_wrapper('message', $message);
    // Comments and nodes are not in the same field on the message entity.
    if (isset($node->node_type) && strpos($node->node_type, 'comment_') !== FALSE) {
      // Comment.
      $wrapper->field_comment_ref->set($node);
    }
    else {
      // Node.
      $wrapper->field_target_nodes[] = $node;
    }
    $wrapper->save();

    return $message;
  }

  /**
   * Wrapper to create and save "moderated_xx" messages.
   *
   * @param object $node
   *   The node to attach the message to. Can be a comment.
   * @param int $moderator_uid
   *   The moderator uid.
   * @param string $message_type
   *   (optional) The type of message to send. If not given,
   *   the function will try to determine it.
   * @param int $uid
   *   (optional) The uid to attach the message to.
   *   If not given, we use the node's author.
   *
   * @return object
   *   The new message object.
   */
  protected function create_moderated_message($node, $moderator_uid, $message_type = NULL, $uid = FALSE) {
    if (!$message_type) {
      // Comments and nodes are not in the same field on the message entity.
      if (isset($node->node_type) && strpos($node->node_type, 'comment_') !== FALSE) {
        // Comment.
        $message_type = 'moderated_comment';
      }
      else {
        // Node.
        $message_type = 'moderated_content';
      }
    }

    $message = message_create($message_type, array('uid' => $uid ? $uid : $node->uid));
    $wrapper = entity_metadata_wrapper('message', $message);
    // Comments and nodes are not in the same field on the message entity.
    if (isset($node->node_type) && strpos($node->node_type, 'comment_') !== FALSE) {
      // Comment.
      $wrapper->field_comment_ref->set($node);
    }
    else {
      // Node.
      $wrapper->field_target_nodes[] = $node;
    }

    // Store the moderator (can sometimes be the same as the user).
    $wrapper->field_moderator = user_load($moderator_uid);

    $wrapper->save();

    return $message;
  }


  /**
   * Creates moderated messages.
   *
   * Creates messages depending on if the node was updated by the user or a
   * moderator.
   *
   * @param string $message_type
   *   The type of message to create if the node was not updated by a moderator.
   * @param object $node
   *   The node to check. $node can be a comment.
   * @param bool $create_message
   *   (optional) If the node is moderated, do we still want to create a
   *   message on the author's profile ?
   */
  protected function messages_moderated($message_type, $node, $create_message = FALSE) {
    if ($uid = $this->is_moderated($node)) {
      // Notify the author but don't create a message on its profile.
      $this->send_mail_author_moderated((isset($node->node_type) && strpos($node->node_type, 'comment_') !== FALSE) ? 'updated_by_moderator_comment' : 'updated_by_moderator', $node, $uid);
      // Create a message on the moderator's profile but no mail.
      $this->create_moderated_message($node, $uid, NULL, $uid);

      if ($create_message) {
        // Create a message on the author's profile.
        $this->create_message($message_type, $node);
      }
    }
    else {
      // Updated by himself, create the message. No mails.
      $this->create_message($message_type, $node);
    }
  }

  /**
   * Checks if a node is being moderated or not.
   *
   * @param object $node
   *   The node to check. $node can be a comment.
   *
   * @return false|int
   *   Returns the moderator UID if it's moderated, otherwise returns FALSE.
   */
  protected function is_moderated($node) {
    global $user;
    // If the node is not updated by its author,
    // then it's updated by a moderator.
    if ($node->uid != $user->uid) {
      return $user->uid;
    }
    return FALSE;
  }

  /**
   * Sends a mail to the author of a moderated content.
   *
   * @param string $message_type
   *   The type of message to send.
   * @param object $node
   *   The node.
   * @param int $moderator_uid
   *   The moderator uid.
   */
  protected function send_mail_author_moderated($message_type, $node, $moderator_uid) {
    $message = $this->create_moderated_message($node, $moderator_uid, $message_type);

    // Let message-notify deliver the email, and send the message for us.
    // We pass in the options the field names, that will be used to capture
    // the rendered message, and provide an email log.
    $options = array(
      'rendered fields' => array(
        'message_notify_email_subject' => 'field_message_rendered_subject',
        'message_notify_email_body' => 'field_message_rendered_body',
      ),
    );

    message_notify_send_message($message, $options);
  }

  /**
   * Create a message in profile and send a mail notification to followers.
   *
   * @param string $message_type
   *   The type of message to send.
   * @param object $node
   *   The node. Can also be a comment (comments are nodes).
   * @param int $nid
   *   The FOLLOWED node id.
   * @param bool $create_message
   *   (optional) If TRUE, we will also create the message to be displayed
   *   on the user's profile.
   */
  protected function node_send_mail_followers($message_type, $node, $nid, $create_message = FALSE) {
    // We can manage the function only if we have the module for.
    if (module_exists('group_mind_follow_node')) {
      include drupal_get_path('module', 'group_mind_follow_node') . '/class/add_function_to_group_mind_messages.php';
    }
  }
}
